// Copyright (c) 2015, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.omnibus_test;

import 'dart:async';

import 'package:omnibus/omnibus.dart';
import 'package:test/test.dart';

class TextEvent extends OmnibusEvent {
  static final String type = 'test.TextEvent';
  String get instanceType => type;

  String data;

  TextEvent(this.data) {}
}

class TextReply extends OmnibusReply {
  static final String type = 'test.TextReply';
  String get instanceType => type;

  String data;

  TextReply(this.data) {}
}

class TextListener extends OmnibusListener {
  bool probe(TextEvent event) {
    return event.data.endsWith('!');
  }

  Future<OmnibusReply> receive(TextEvent event) async {
    var completer = new Completer<OmnibusReply>();

    new Timer(
        new Duration(milliseconds: 100),
        () => completer.complete(
            new TextReply(event.data.substring(0, event.data.length - 1))));

    return completer.future;
  }
}

main() {
  test('Omnibus await replies', () async {
    var bus = new Omnibus();

    bus.subscribe(TextEvent.type, new TextListener());

    var replies = await bus.emit(new TextEvent('Hello World!'));

    expect(replies.single.data, equals("Hello World"));
  });
}
